#include "htab.h"

/*********************************************/
/* DEL_HTABS - clear out our hash tables     */
/*********************************************/
/* ؽ ̺    */
void del_htabs()
{
	del_elist(ems_htab);			/* Clear out our various    */
}

ENODEPTR new_enode(char *userId, char *to)
{
	ENODEPTR	newptr;
	char		*sptr_userId;
	char		*sptr_to;

	//printf("\t%s \t %s", userId, to);
	if (strlen(userId) >= MAXID)
	{
		fprintf(stderr,"[new_enode] %s (%d)\n","Big ",strlen(userId));
		userId[MAXID-1]=0;
	}
	if ( (sptr_userId=(char *)memset((char *)malloc(strlen(userId)+1),0x00, strlen(userId)+1))==NULL ) return (ENODEPTR)NULL;
//	if ( (sptr_userId=malloc(strlen(userId)+1))==NULL ) return (ENODEPTR)NULL;
	//if ( (sptr_userId=malloc(strlen(userId)))==NULL ) return (ENODEPTR)NULL;
	memcpy(sptr_userId,userId,strlen(userId));
	if (strlen(to) >= MAXSTR)
	{
		fprintf(stderr,"[new_enode] %s (%d)\n","Big ",strlen(to));
		to[MAXSTR-1]=0;
	}
	if ( (sptr_to=(char *)memset((char *)malloc(strlen(to)+1),0x00, strlen(to)+1))==NULL ) return (ENODEPTR)NULL;
	//if ( (sptr_to=malloc(strlen(to)+1))==NULL ) return (ENODEPTR)NULL;
	//if ( (sptr_to=malloc(strlen(to)))==NULL ) return (ENODEPTR)NULL;
	memcpy(sptr_to,to, strlen(to));

	if (( newptr = malloc(sizeof(struct enode))) != NULL)
	{
		newptr->userId  = sptr_userId;
		newptr->to		= sptr_to;
	} else
	{
		free(sptr_userId);
		free(sptr_to);
	}
	//printf("\t%s \t %s\n", newptr->userId, newptr->to);
	return newptr;
}

/*********************************************/
/* PUT_ENODE - insert/update email node       */
/*********************************************/

int put_enode(	char	*userId,
				char	*to,   /* STRING  */
				ENODEPTR *htab)  /* ptr>next  */
{
	ENODEPTR cptr,nptr;

	/* check if hashed */
	if ( (cptr = htab[hash(userId)]) == NULL)
	{
		/* not hashed */
		if ( (nptr=new_enode(userId, to)) != NULL)
		{
			nptr->next = NULL;
			htab[hash(userId)] = nptr;
		}
	} else
	{
		/* hashed */
		while (cptr != NULL)
		{
			cptr = cptr->next;
		}
		/* not found... */
		if ( (nptr = new_enode(userId, to)) != NULL)
		{
			nptr->next  = htab[hash(userId)];
			htab[hash(userId)]=nptr;
		}
	}
	return nptr==NULL;
}

/*********************************************/
/* DEL_ELIST - delete email hash table        */
/*********************************************/

void	del_elist(ENODEPTR *htab)
{
	/* free memory used by hash table */
	ENODEPTR aptr,temp;
	int i;

	for (i=0;i<MAXHASH;i++)
	{
		if (htab[i] != NULL)
		{
			aptr = htab[i];
			while (aptr != NULL)
			{
				temp = aptr->next;
				free (aptr->userId);   /* free email string space */
				free (aptr->to);   /* free email string space */
				free (aptr);        /* free email structure    */
				aptr = temp;
			}
			htab[i]=NULL;
		}
	}
}

/*********************************************/
/* DEL_SPLIST - delete specific email hash table        */
/*********************************************/

void	del_splist(ENODEPTR *htab, int start, int end)
{
	/* free memory used by hash table */
	ENODEPTR aptr,temp;
	int i;

	for (i=start;i<=end;i++)
	{
		if (htab[i] != NULL)
		{
			aptr = htab[i];
			while (aptr != NULL)
			{
				temp = aptr->next;
				free (aptr->userId);   /* free email string space */
				free (aptr->to);   /* free email string space */
				free (aptr);        /* free email structure    */
				aptr = temp;
			}
			htab[i]=NULL;
		}
	}
}

/*********************************************/
/* DEL_bucket - delete a email hash table        */
/*********************************************/
void del_bucket(ENODEPTR htab)
{
	/* free memory for one hash table */
	ENODEPTR aptr, temp;
	if(htab!=NULL)
	{
		aptr = htab;
		while(aptr != NULL)
		{
			temp = aptr->next;
			free (aptr->userId);   /* free email string space */
			free(aptr->to);   /* free email string space */
			free(aptr);
			aptr = temp;
		}
		htab = NULL;
	}

}

/*********************************************/
/* HASH - return hash value for string       */
/*********************************************/

u_long hash(char *str)
{
	u_long hashval;
	for (hashval = 0; *str != '\0'; str++)
		hashval = *str + 31 * hashval;
	return hashval % MAXHASH;
}
/********************************************/
/* HASH - init HASH Table 				    */
/********************************************/
void hash_init()
{
	int i;
	for(i=0;i<MAXHASH;i++)
	{
		ems_htab[i]=NULL; /* initialize */
	}
}
